@@ -29,6 +29,41 @@
 #include <sys/stat.h>
 #include <limits.h>
 
+static char *rval_open_name(snd_use_case_mgr_t *uc_mgr)
+{
+	const char *name;
+	if (uc_mgr->conf_format < 3)
+		return NULL;
+	name = uc_mgr->card_name;
+	if (name) {
+		if (strncmp(name, "strict:", 7) == 0)
+			name += 7;
+		return strdup(name);
+	}
+	return NULL;
+}
+
+static char *rval_conf_topdir(snd_use_case_mgr_t *uc_mgr)
+{
+	const char *dir;
+
+	if (uc_mgr->conf_format < 3)
+		return NULL;
+	dir = uc_mgr_config_dir(uc_mgr->conf_format);
+	if (dir && dir[0])
+		return strdup(dir);
+	return NULL;
+}
+
+static char *rval_conf_dir(snd_use_case_mgr_t *uc_mgr)
+{
+	if (uc_mgr->conf_format < 3)
+		return NULL;
+	if (uc_mgr->conf_dir_name && uc_mgr->conf_dir_name[0])
+		return strdup(uc_mgr->conf_dir_name);
+	return NULL;
+}
+
 static char *rval_conf_name(snd_use_case_mgr_t *uc_mgr)
 {
 	if (uc_mgr->conf_file_name && uc_mgr->conf_file_name[0])
@@ -36,11 +71,25 @@ static char *rval_conf_name(snd_use_case_mgr_t *uc_mgr)
 	return NULL;
 }
 
+static char *rval_card_number(snd_use_case_mgr_t *uc_mgr)
+{
+	struct ctl_list *ctl_list;
+	char num[16];
+
+	if (uc_mgr->conf_format < 3)
+		return NULL;
+	ctl_list = uc_mgr_get_master_ctl(uc_mgr);
+	if (ctl_list == NULL)
+		return strdup("");
+	snprintf(num, sizeof(num), "%i", snd_ctl_card_info_get_card(ctl_list->ctl_info));
+	return strdup(num);
+}
+
 static char *rval_card_id(snd_use_case_mgr_t *uc_mgr)
 {
 	struct ctl_list *ctl_list;
 
-	ctl_list = uc_mgr_get_one_ctl(uc_mgr);
+	ctl_list = uc_mgr_get_master_ctl(uc_mgr);
 	if (ctl_list == NULL)
 		return NULL;
 	return strdup(snd_ctl_card_info_get_id(ctl_list->ctl_info));
@@ -50,7 +99,7 @@ static char *rval_card_driver(snd_use_case_mgr_t *uc_mgr)
 {
 	struct ctl_list *ctl_list;
 
-	ctl_list = uc_mgr_get_one_ctl(uc_mgr);
+	ctl_list = uc_mgr_get_master_ctl(uc_mgr);
 	if (ctl_list == NULL)
 		return NULL;
 	return strdup(snd_ctl_card_info_get_driver(ctl_list->ctl_info));
@@ -60,7 +109,7 @@ static char *rval_card_name(snd_use_case_mgr_t *uc_mgr)
 {
 	struct ctl_list *ctl_list;
 
-	ctl_list = uc_mgr_get_one_ctl(uc_mgr);
+	ctl_list = uc_mgr_get_master_ctl(uc_mgr);
 	if (ctl_list == NULL)
 		return NULL;
 	return strdup(snd_ctl_card_info_get_name(ctl_list->ctl_info));
@@ -70,7 +119,7 @@ static char *rval_card_longname(snd_use_case_mgr_t *uc_mgr)
 {
 	struct ctl_list *ctl_list;
 
-	ctl_list = uc_mgr_get_one_ctl(uc_mgr);
+	ctl_list = uc_mgr_get_master_ctl(uc_mgr);
 	if (ctl_list == NULL)
 		return NULL;
 	return strdup(snd_ctl_card_info_get_longname(ctl_list->ctl_info));
@@ -80,12 +129,37 @@ static char *rval_card_components(snd_use_case_mgr_t *uc_mgr)
 {
 	struct ctl_list *ctl_list;
 
-	ctl_list = uc_mgr_get_one_ctl(uc_mgr);
+	ctl_list = uc_mgr_get_master_ctl(uc_mgr);
 	if (ctl_list == NULL)
 		return NULL;
 	return strdup(snd_ctl_card_info_get_components(ctl_list->ctl_info));
 }
 
+static char *rval_card_id_by_name(snd_use_case_mgr_t *uc_mgr, const char *id)
+{
+	struct ctl_list *ctl_list;
+	char *name, *index;
+	long idx = 0;
+
+	if (uc_mgr->conf_format < 3) {
+		uc_error("CardIdByName substitution is supported in v3+ syntax");
+		return NULL;
+	}
+
+	name = alloca(strlen(id) + 1);
+	strcpy(name, id);
+	index = strchr(name, '#');
+	if (index) {
+		*index = '\0';
+		if (safe_strtol(index + 1, &idx))
+			return NULL;
+	}
+	ctl_list = uc_mgr_get_ctl_by_name(uc_mgr, name, idx);
+	if (ctl_list == NULL)
+		return NULL;
+	return strdup(snd_ctl_card_info_get_id(ctl_list->ctl_info));
+}
+
 static char *rval_env(snd_use_case_mgr_t *uc_mgr ATTRIBUTE_UNUSED, const char *id)
 {
 	char *e;
@@ -146,6 +220,21 @@ static char *rval_sysfs(snd_use_case_mgr_t *uc_mgr ATTRIBUTE_UNUSED, const char
 	return strdup(path);
 }
 
+static char *rval_var(snd_use_case_mgr_t *uc_mgr, const char *id)
+{
+	const char *v;
+
+	if (uc_mgr->conf_format < 3) {
+		uc_error("variable substitution is supported in v3+ syntax");
+		return NULL;
+	}
+
+	v = uc_mgr_get_variable(uc_mgr, id);
+	if (v)
+		return strdup(v);
+	return NULL;
+}
+
 #define MATCH_VARIABLE(name, id, fcn, empty_ok)				\
 	if (strncmp((name), (id), sizeof(id) - 1) == 0) { 		\
 		rval = fcn(uc_mgr);					\
@@ -193,7 +282,11 @@ int uc_mgr_get_substituted_value(snd_use_case_mgr_t *uc_mgr,
 		if (*value == '$' && *(value+1) == '{') {
 			bool allow_empty = false;
 
+			MATCH_VARIABLE(value, "${OpenName}", rval_open_name, false);
+			MATCH_VARIABLE(value, "${ConfTopDir}", rval_conf_topdir, false);
+			MATCH_VARIABLE(value, "${ConfDir}", rval_conf_dir, false);
 			MATCH_VARIABLE(value, "${ConfName}", rval_conf_name, false);
+			MATCH_VARIABLE(value, "${CardNumber}", rval_card_number, true);
 			MATCH_VARIABLE(value, "${CardId}", rval_card_id, false);
 			MATCH_VARIABLE(value, "${CardDriver}", rval_card_driver, false);
 			MATCH_VARIABLE(value, "${CardName}", rval_card_name, false);
@@ -201,6 +294,8 @@ int uc_mgr_get_substituted_value(snd_use_case_mgr_t *uc_mgr,
 			MATCH_VARIABLE(value, "${CardComponents}", rval_card_components, true);
 			MATCH_VARIABLE2(value, "${env:", rval_env);
 			MATCH_VARIABLE2(value, "${sys:", rval_sysfs);
+			MATCH_VARIABLE2(value, "${var:", rval_var);
+			MATCH_VARIABLE2(value, "${CardIdByName:", rval_card_id_by_name);
 			err = -EINVAL;
 			tmp = strchr(value, '}');
 			if (tmp) {
